package org.bitbucket.bradleysmithllc.etlagent.informatica.handler;

import com.fasterxml.jackson.databind.JsonNode;
import com.google.gson.stream.JsonWriter;
import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.etlagent.dto.GenericResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.ExecuteWorkflowRequestDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.ExecuteWorkflowResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.RemoteFile;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.*;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.util.VelocityUtil;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

public class ExecuteWorkflow extends AbstractHandler<ExecuteWorkflowRequestDTO, GenericResponseDTO> {
	Map<String, String> velocityContext = new HashMap<String, String>();

	@Override
	public String getId() {
		return "executeWorkflow";
	}

	@Override
	public ExecuteWorkflowRequestDTO getRequestContainerObject() {
		return new ExecuteWorkflowRequestDTO("", "");
	}

	@Override
	public GenericResponseDTO process(JsonNode request, final ExecuteWorkflowRequestDTO container) throws Exception {
		System.out.println("Executing workflow");

		String domain = container.getInformaticaDomain();

		String repo = container.getInformaticaRepository();
		String hub = container.getInformaticaWebServicesHub();

		String intsvc = container.getInformaticaIntegrationService();

		// resolve to an informatica domain
		InformaticaDomain informaticaDomain = informaticaConfiguration.getDefaultDomain();

		if (domain != null) {
			System.out.println("No domain specified - using the default domain");

			informaticaDomain = informaticaConfiguration.getDomain(domain);

			if (informaticaDomain == null) {
				throw new IllegalArgumentException("Domain '" + domain + "' not found");
			}
		} else {
			System.out.println("No domain specified - using the default domain " + informaticaDomain.getDomainName());
		}

		InformaticaRepository informaticaRepository = null;

		if (repo == null) {
			informaticaRepository = informaticaDomain.getDefaultRepository();
		} else {
			informaticaRepository = informaticaDomain.getRepository(repo);
		}

		if (informaticaRepository == null) {
			throw new IllegalArgumentException("Domain '" + domain + "' does not contain repository '" + repo + "'");
		}

		InformaticaIntegrationService informaticaIntegrationService = informaticaDomain.getDefaultIntegrationService();

		if (intsvc != null) {
			informaticaIntegrationService = informaticaDomain.getIntegrationService(intsvc);
		}

		if (informaticaIntegrationService == null) {
			throw new IllegalArgumentException("Repository '" + informaticaRepository.getQualifiedName() + "' does not contain integration service '" + intsvc + "'");
		}

		InformaticaWebServicesHub informaticaWebServicesHub = informaticaRepository.getDefaultWebServicesHub();

		if (hub != null)
		{
			informaticaWebServicesHub = informaticaRepository.getWebServicesHub(hub);

			if (informaticaWebServicesHub == null) {
				throw new IllegalArgumentException("Repository '" + informaticaRepository.getQualifiedName() + "' does not contain web services hub '" + hub + "'");
			}
		}

		// prepare the workspace
		prepareWorkspace(informaticaDomain, container);

		final ExecuteWorkflowResponseDTO executeWorkflowResponseDTO = new ExecuteWorkflowResponseDTO();

		useExecutionService(informaticaRepository, informaticaIntegrationService, informaticaWebServicesHub, new IntegrationServiceVisitor() {
			@Override
			public void useClient(InformaticaExecutionClient client, RuntimeSupport runtimeSupport) throws Exception {
				String tag = container.getFolder() + "." + container.getWorkflowName() + (container.getTask() != null ? ("." + container.getTask()) : "") + (container.getRunInstanceName() != null ? ("." + container.getRunInstanceName()) : "");

				ParameterFile pfile = container.getParameterFile();

				File parmFile = runtimeSupport.createTempFile(tag + ".PRM");

				try {
					PrintWriter fw = new PrintWriter(new BufferedWriter(new FileWriter(parmFile)));

					try {
						pfile.write(fw);
					} finally {
						fw.close();
					}

					// rework velocity references
					String parmContents = FileUtils.readFileToString(parmFile);

					parmContents = parmContents.replaceAll("#[\\s]*\\{", "\\$\\{");

					FileUtils.write(parmFile, VelocityUtil.writeTemplate(parmContents, velocityContext));
					// reload to get the updated values
					pfile = new ParameterFile(parmFile);
				} finally {
					parmFile.delete();
				}

				InformaticaExecutionResult execResult;

				if (container.getTask() != null) {
					if (container.getRunInstanceName() != null) {
						execResult = client.executeWorkflowTask(
								container.getWorkflowName(),
								container.getRunInstanceName(),
								container.getTask(),
								container.getFolder(),
								pfile,
								container.getRequestFiles()
						);
					} else {
						execResult = client.executeWorkflowTask(
								container.getWorkflowName(),
								container.getTask(),
								container.getFolder(),
								pfile,
								container.getRequestFiles()
						);
					}
				} else {
					if (container.getRunInstanceName() != null) {
						execResult = client.executeWorkflow(
								container.getWorkflowName(),
								container.getRunInstanceName(),
								container.getFolder(),
								pfile,
								container.getRequestFiles()
						);
					} else {
						execResult = client.executeWorkflow(
								container.getWorkflowName(),
								container.getFolder(),
								pfile,
								container.getRequestFiles()
						);
					}
				}

				executeWorkflowResponseDTO.getResponseFiles().putAll(execResult.getWorkSpaceFiles());

				if (execResult.getFailureExc() != null)
				{
					executeWorkflowResponseDTO.setInformaticaErrorMessage(execResult.getFailureExc());
				}
			}
		});

		// wipe out the temporary workspace
		FileUtils.forceDelete(informaticaDomain.getLocalizedWorkingRoot());

		return executeWorkflowResponseDTO;
	}

	private void prepareWorkspace(InformaticaDomain domain, ExecuteWorkflowRequestDTO container) throws IOException {
		velocityContext.clear();

		RemoteFile.copyRemoteToLocal(container.getRequestFiles(), domain);

		for (RemoteFile.fileType type : RemoteFile.fileType.values()) {
			File targetFiles = domain.getWorkspaceDir(type);
			velocityContext.put(type.name(), targetFiles.getAbsolutePath());
		}
	}
}